@@ -3,27 +3,22 @@ use bevy::ecs::prelude::*;
 use bevy::math::Vec3;
 use crossbeam::channel::Receiver;
 
-use heron_core::{CollisionEvent, Gravity, PhysicsTime};
+use heron_core::{CollisionData, CollisionEvent, Gravity, PhysicsSteps, PhysicsTime};
 
-use crate::convert::IntoRapier;
+use crate::convert::{IntoBevy, IntoRapier};
 use crate::rapier::dynamics::{CCDSolver, IntegrationParameters, JointSet, RigidBodySet};
 use crate::rapier::geometry::{
     BroadPhase, ColliderHandle, ColliderSet, ContactEvent, IntersectionEvent, NarrowPhase,
 };
 use crate::rapier::pipeline::{ChannelEventCollector, PhysicsPipeline};
 
-#[derive(Copy, Clone)]
-pub(crate) struct PhysicsStepPerSecond(pub(crate) f32);
-
 pub(crate) fn update_integration_parameters(
-    steps_per_second: Option<Res<'_, PhysicsStepPerSecond>>,
+    physics_steps: Res<'_, PhysicsSteps>,
     physics_time: Res<'_, PhysicsTime>,
     mut integration_parameters: ResMut<'_, IntegrationParameters>,
 ) {
-    if let Some(steps_per_second) = steps_per_second {
-        if steps_per_second.is_changed() {
-            integration_parameters.dt = physics_time.scale() / steps_per_second.0;
-        }
+    if physics_steps.is_changed() || physics_time.is_changed() {
+        integration_parameters.dt = physics_steps.duration().as_secs_f32() * physics_time.scale();
     }
 }
 
@@ -55,7 +50,7 @@ pub(crate) fn step(
         &event_manager.handler,
     );
 
-    event_manager.fire_events(&colliders, &mut events);
+    event_manager.fire_events(&bodies, &colliders, &mut events);
 }
 
 pub(crate) struct EventManager {
@@ -78,17 +73,22 @@ impl Default for EventManager {
 }
 
 impl EventManager {
-    fn fire_events(&self, colliders: &ColliderSet, events: &mut Events<CollisionEvent>) {
+    fn fire_events(
+        &self,
+        bodies: &RigidBodySet,
+        colliders: &ColliderSet,
+        events: &mut Events<CollisionEvent>,
+    ) {
         while let Ok(event) = self.contacts.try_recv() {
             match event {
                 ContactEvent::Started(h1, h2) => {
-                    if let Some((e1, e2)) = Self::entity_pair(colliders, h1, h2) {
-                        events.send(CollisionEvent::Started(e1, e2));
+                    if let Some((d1, d2)) = Self::data(bodies, colliders, h1, h2) {
+                        events.send(CollisionEvent::Started(d1, d2));
                     }
                 }
                 ContactEvent::Stopped(h1, h2) => {
-                    if let Some((e1, e2)) = Self::entity_pair(colliders, h1, h2) {
-                        events.send(CollisionEvent::Stopped(e1, e2));
+                    if let Some((d1, d2)) = Self::data(bodies, colliders, h1, h2) {
+                        events.send(CollisionEvent::Stopped(d1, d2));
                     }
                 }
             }
@@ -100,7 +100,7 @@ impl EventManager {
             intersecting,
         }) = self.intersections.try_recv()
         {
-            if let Some((e1, e2)) = Self::entity_pair(colliders, collider1, collider2) {
+            if let Some((e1, e2)) = Self::data(bodies, colliders, collider1, collider2) {
                 if intersecting {
                     events.send(CollisionEvent::Started(e1, e2));
                 } else {
@@ -111,15 +111,39 @@ impl EventManager {
     }
 
     #[allow(clippy::cast_possible_truncation)]
-    fn entity_pair(
+    fn data(
+        bodies: &RigidBodySet,
         colliders: &ColliderSet,
         h1: ColliderHandle,
         h2: ColliderHandle,
-    ) -> Option<(Entity, Entity)> {
-        if let (Some(b1), Some(b2)) = (colliders.get(h1), colliders.get(h2)) {
-            let e1 = Entity::from_bits(b1.user_data as u64);
-            let e2 = Entity::from_bits(b2.user_data as u64);
-            Some(if e1 < e2 { (e1, e2) } else { (e2, e2) })
+    ) -> Option<(CollisionData, CollisionData)> {
+        if let (Some(collider1), Some(collider2)) = (colliders.get(h1), colliders.get(h2)) {
+            if let (Some(rb1), Some(rb2)) = (
+                bodies.get(collider1.parent()),
+                bodies.get(collider2.parent()),
+            ) {
+                let d1 = CollisionData::new(
+                    Entity::from_bits(rb1.user_data as u64),
+                    Entity::from_bits(collider1.user_data as u64),
+                    collider1.collision_groups().into_bevy(),
+                );
+                let d2 = CollisionData::new(
+                    Entity::from_bits(rb2.user_data as u64),
+                    Entity::from_bits(collider2.user_data as u64),
+                    collider2.collision_groups().into_bevy(),
+                );
+                Some(
+                    if Entity::from_bits(rb1.user_data as u64)
+                        < Entity::from_bits(rb2.user_data as u64)
+                    {
+                        (d1, d2)
+                    } else {
+                        (d2, d1)
+                    },
+                )
+            } else {
+                None
+            }
         } else {
             None
         }
@@ -128,6 +152,8 @@ impl EventManager {
 
 #[cfg(test)]
 mod tests {
+    use heron_core::CollisionLayers;
+
     use crate::pipeline::EventManager;
     use crate::rapier::dynamics::RigidBodyBuilder;
     use crate::rapier::geometry::{ColliderBuilder, ColliderHandle};
@@ -136,9 +162,14 @@ mod tests {
     use super::*;
 
     struct TestContext {
+        bodies: RigidBodySet,
         colliders: ColliderSet,
-        entity1: Entity,
-        entity2: Entity,
+        rb_entity_1: Entity,
+        rb_entity_2: Entity,
+        collider_entity_1: Entity,
+        collider_entity_2: Entity,
+        layers_1: CollisionLayers,
+        layers_2: CollisionLayers,
         handle1: ColliderHandle,
         handle2: ColliderHandle,
     }
@@ -148,29 +179,48 @@ mod tests {
             let mut bodies = RigidBodySet::new();
             let mut colliders = ColliderSet::new();
 
-            let entity1 = Entity::new(0);
-            let entity2 = Entity::new(1);
-            let body1 = bodies.insert(RigidBodyBuilder::new_dynamic().build());
-            let body2 = bodies.insert(RigidBodyBuilder::new_dynamic().build());
+            let rb_entity_1 = Entity::new(0);
+            let rb_entity_2 = Entity::new(1);
+            let collider_entity_1 = Entity::new(2);
+            let collider_entity_2 = Entity::new(3);
+            let layers_1 = CollisionLayers::from_bits(1, 2);
+            let layers_2 = CollisionLayers::from_bits(3, 4);
+            let body1 = bodies.insert(
+                RigidBodyBuilder::new_dynamic()
+                    .user_data(rb_entity_1.to_bits().into())
+                    .build(),
+            );
+            let body2 = bodies.insert(
+                RigidBodyBuilder::new_dynamic()
+                    .user_data(rb_entity_2.to_bits().into())
+                    .build(),
+            );
             let handle1 = colliders.insert(
                 ColliderBuilder::ball(1.0)
-                    .user_data(entity1.to_bits().into())
+                    .user_data(collider_entity_1.to_bits().into())
+                    .collision_groups(layers_1.into_rapier())
                     .build(),
                 body1,
                 &mut bodies,
             );
             let handle2 = colliders.insert(
                 ColliderBuilder::ball(1.0)
-                    .user_data(entity2.to_bits().into())
+                    .user_data(collider_entity_2.to_bits().into())
+                    .collision_groups(layers_2.into_rapier())
                     .build(),
                 body2,
                 &mut bodies,
             );
 
             Self {
+                bodies,
                 colliders,
-                entity1,
-                entity2,
+                rb_entity_1,
+                rb_entity_2,
+                collider_entity_1,
+                collider_entity_2,
+                layers_1,
+                layers_2,
                 handle1,
                 handle2,
             }
@@ -187,12 +237,15 @@ mod tests {
             .handle_contact_event(ContactEvent::Started(context.handle1, context.handle2));
 
         let mut events = Events::<CollisionEvent>::default();
-        manager.fire_events(&context.colliders, &mut events);
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
         let events: Vec<CollisionEvent> = events.get_reader().iter(&events).copied().collect();
 
+        assert_eq!(events.len(), 1);
+        let event = events[0];
+        assert!(matches!(event, CollisionEvent::Started(_, _)));
         assert_eq!(
-            events,
-            vec![CollisionEvent::Started(context.entity1, context.entity2),]
+            event.collision_shape_entities(),
+            (context.collider_entity_1, context.collider_entity_2)
         );
     }
 
@@ -206,12 +259,15 @@ mod tests {
             .handle_contact_event(ContactEvent::Stopped(context.handle1, context.handle2));
 
         let mut events = Events::<CollisionEvent>::default();
-        manager.fire_events(&context.colliders, &mut events);
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
         let events: Vec<CollisionEvent> = events.get_reader().iter(&events).copied().collect();
 
+        assert_eq!(events.len(), 1);
+        let event = events[0];
+        assert!(matches!(event, CollisionEvent::Stopped(_, _)));
         assert_eq!(
-            events,
-            vec![CollisionEvent::Stopped(context.entity1, context.entity2),]
+            event.collision_shape_entities(),
+            (context.collider_entity_1, context.collider_entity_2)
         );
     }
 
@@ -229,12 +285,15 @@ mod tests {
             ));
 
         let mut events = Events::<CollisionEvent>::default();
-        manager.fire_events(&context.colliders, &mut events);
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
         let events: Vec<CollisionEvent> = events.get_reader().iter(&events).copied().collect();
 
+        assert_eq!(events.len(), 1);
+        let event = events[0];
+        assert!(matches!(event, CollisionEvent::Started(_, _)));
         assert_eq!(
-            events,
-            vec![CollisionEvent::Started(context.entity1, context.entity2),]
+            event.collision_shape_entities(),
+            (context.collider_entity_1, context.collider_entity_2)
         );
     }
 
@@ -252,12 +311,59 @@ mod tests {
             ));
 
         let mut events = Events::<CollisionEvent>::default();
-        manager.fire_events(&context.colliders, &mut events);
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
         let events: Vec<CollisionEvent> = events.get_reader().iter(&events).copied().collect();
 
+        assert_eq!(events.len(), 1);
+        let event = events[0];
+        assert!(matches!(event, CollisionEvent::Stopped(_, _)));
         assert_eq!(
-            events,
-            vec![CollisionEvent::Stopped(context.entity1, context.entity2)]
+            event.collision_shape_entities(),
+            (context.collider_entity_1, context.collider_entity_2)
+        );
+    }
+
+    #[test]
+    fn contains_rigid_body_entities() {
+        let manager = EventManager::default();
+        let context = TestContext::default();
+
+        manager
+            .handler
+            .handle_contact_event(ContactEvent::Started(context.handle1, context.handle2));
+
+        let mut events = Events::<CollisionEvent>::default();
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
+        assert_eq!(
+            events
+                .get_reader()
+                .iter(&events)
+                .next()
+                .unwrap()
+                .rigid_body_entities(),
+            (context.rb_entity_1, context.rb_entity_2)
+        );
+    }
+
+    #[test]
+    fn contains_collision_layers() {
+        let manager = EventManager::default();
+        let context = TestContext::default();
+
+        manager
+            .handler
+            .handle_contact_event(ContactEvent::Started(context.handle1, context.handle2));
+
+        let mut events = Events::<CollisionEvent>::default();
+        manager.fire_events(&context.bodies, &context.colliders, &mut events);
+        assert_eq!(
+            events
+                .get_reader()
+                .iter(&events)
+                .next()
+                .unwrap()
+                .collision_layers(),
+            (context.layers_1, context.layers_2)
         );
     }
 }
