@@ -5,14 +5,15 @@
 
 use std::f32::consts::PI;
 use std::ops::DerefMut;
+use std::time::Duration;
 
 use bevy::core::CorePlugin;
 use bevy::prelude::*;
 use bevy::reflect::TypeRegistryArc;
 
-use heron_core::{CollisionShape, RigidBody};
+use heron_core::{CollisionShape, PhysicsSteps, RigidBody};
 use heron_rapier::convert::IntoBevy;
-use heron_rapier::rapier::dynamics::{IntegrationParameters, RigidBodyHandle, RigidBodySet};
+use heron_rapier::rapier::dynamics::{RigidBodyHandle, RigidBodySet};
 use heron_rapier::rapier::geometry::{ColliderHandle, ColliderSet};
 use heron_rapier::RapierPlugin;
 
@@ -20,12 +21,9 @@ fn test_app() -> App {
     let mut builder = App::build();
     builder
         .init_resource::<TypeRegistryArc>()
+        .insert_resource(PhysicsSteps::every_frame(Duration::from_secs(1)))
         .add_plugin(CorePlugin)
-        .add_plugin(RapierPlugin {
-            step_per_second: None,
-            parameters: IntegrationParameters::default(),
-        });
-
+        .add_plugin(RapierPlugin);
     builder.app
 }
 
